unit ClipNote;

{The event OnClipboardChange is triggered,
 when any application changes clipboard data.

 by Sergei V. Trepalin, trep@ism.ac.ru
 Free. No warranty.}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TClipNote = class(TComponent)
  private
    { Private declarations }
    FNextChain:THandle;
    FOnClipboardChange:TNotifyEvent;
    FOldProc,FNewProc:pointer;
    procedure UnhookControl;
    procedure HookProc(var Message:TMessage);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property OnClipboardChange:TNotifyEvent read FOnClipboardChange write FOnClipboardChange;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TClipNote]);
end;

constructor TClipNote.Create(AOwner: TComponent);
var
  N:integer;
begin
  inherited;
  UnhookControl;
  if (Owner<>nil) and not (csDesigning in ComponentState) then begin
    N:=GetWindowLong((Owner as TWinControl).Handle,GWL_WNDPROC);
    FOldProc:=pointer(N);
    FNewProc:=MakeObjectInstance(HookProc);
    N:=integer(FNewProc);
    SetWindowLong((Owner as TWinControl).Handle,GWL_WNDPROC,N);
    FNextChain:=SetClipboardViewer((AOwner as TWinControl).Handle);
  end;
end;

destructor TClipNote.Destroy;
begin
  UnhookControl;
  inherited;
end;

procedure TClipNote.HookProc(var Message: TMessage);
begin
  if Message.Msg=WM_DRAWCLIPBOARD then begin
    if Assigned(FOnClipboardChange) then FOnClipboardChange(Self);
    Message.Result:=0;
  end else if Message.Msg=WM_CHANGECBCHAIN then begin
    if Message.wParam=FNextChain then FNextChain:=Message.lParam else
    if FNextChain<>0 then SendMessage(FNextChain,Message.Msg,Message.wParam,Message.lParam);
    Message.Result:=0;
  end;
  Message.Result:=CallWindowProc(FOldProc,(Owner as TWinControl).Handle,Message.Msg,
  Message.wParam,Message.lParam);
  if Message.Msg=WM_DESTROY then begin
    UnhookControl;
    ChangeClipboardChain((Owner as TWinControl).Handle,FNextChain);
  end;
end;

procedure TClipNote.UnhookControl;
var
  N:integer;
begin
  if (Owner<>nil) and (FOldProc<>nil) and (FNewProc<>nil) then begin
    N:=integer(FOldProc);
    SetWindowLong((Owner as TWinControl).Handle,GWL_WNDPROC,N);
    FreeObjectInstance(FNewProc);
    FOldProc:=nil;
    FNewProc:=nil;
  end;
end;

end.
